#!/usr/bin/env python
# -*- coding: UTF-8 -*-

# 市场代码
SH = 1  # 市场代码：上海
SZ = 2  # 市场代码：深圳
CFFEX = 5  # 市场代码：中金所
SHFE = 6  # 市场代码：上期所
DEC = 7  # 市场代码：郑商所
CZCE = 8  # 市场代码：郑商所
INE = 21  # 市场代码：上海国际能源交易中心

# 周期
MS = 1
S = 1000
M = 60 * S
H = 60 * M
D = 24 * H

# 行情类型
STOCK = 1
FUTURE = 2
OPTION = 3
INDEX = 4
